
//
// Level compressor
//

#include <windows.h>
#include <stdio.h>
#include <string.h>

#include "levels.h"



static int readInt(const unsigned char *data, int numdigits, int base) {
  int i;
  int result=0;

  for(i=0;i<numdigits;i++) {
    result=result*base+((data[i]<='9')?(data[i]-'0'):((data[i]&7)+9));
  }
  return result;
}

static int levelX;
static int levelY;
static int diamondsNeeded;
static int maxAmoebaCount;
static int colors[4];
static int level1offs;

static const unsigned char objs[16]={
  ' ', '.', 'r', 'd', 'w', 'W', 'x', 'X',
  'a', 'B', 'E', 'f', 'b', 0,   0,   0};

int main(int argc, char **argv) {
  int i,x,y;
  int prevCode=-1;
  int codeCount=0;
  int offs=0;
  int totalOffs=0;
  int linelen;
  int curLevel=0;

  FILE *lf=fopen("compressedlevels.h", "wt");
  if (lf) {
    fprintf(lf, "static const unsigned char levelData[]={\n");
    while (levels[curLevel]) {
      const unsigned char *levelData=levels[curLevel];

      levelX=readInt(levelData, 3, 10);
      levelY=readInt(levelData+4, 3, 10);
      diamondsNeeded=readInt(levelData+8, 3, 10);

      fprintf(lf, "0x%02X,0x%02X,  0x%02X,0x%02X,\n",
        levelX, levelY,
        (diamondsNeeded>>8), (diamondsNeeded&255));
      for(i=0;i<4;i++) {
        int color=readInt(levelData+12+i*7, 6, 16);
        fprintf(lf, "0x%02X,0x%02X,0x%02X,\n", (color>>16)&255, (color>>8)&255, color&255);
      }

      offs+=4+3*4;
      linelen=0;
      levelData+=39;

      for(y=0;y<levelY-2;y++) {
        for(x=0;x<levelX-2;x++) {
          unsigned char c=0;
          for(i=0;i<16;i++) {
            if (objs[i]==*levelData) {
              c=i;
            }
          }
          if (c==prevCode && codeCount<255) {
            codeCount++;
          } else {
            if (prevCode>=0) {
              if (codeCount<16) {
                fprintf(lf, "0x%02X,  ", (codeCount<<4)+prevCode);
                offs++;
                linelen++;
              } else {
                fprintf(lf, "0x%02X,0x%02X,  ", prevCode, codeCount);
                offs+=2;
                linelen+=2;
              }
              if (linelen>8) {
                fprintf(lf, "\n"); linelen=0;
              }
            }
            codeCount=1;
            prevCode=c;
          }

          levelData++;
        }
      }
      if (prevCode>=0) {
        if (codeCount<16) {
          fprintf(lf, "0x%02X,", (codeCount<<4)+prevCode);
          offs++;
        } else {
          fprintf(lf, "0x%02X,0x%02X,", prevCode, codeCount);
          offs+=2;
        }
      }
      prevCode=-1;

      fprintf(lf, "\n\n");
      if (curLevel==0) level1offs=offs;
      printf("Level %d: %dx%d -> %d objects compressed to %d data elements.\n", curLevel, levelX, levelY, levelX*levelY, offs);
      totalOffs+=offs;
      offs=0;

      curLevel++;
    }
    fprintf(lf, "0x00\n};\n#define LEVEL1OFFS %d\n", level1offs);
    fclose(lf);
    printf("LevelCompressor: 'compressedlevels.h' with %d data elements generated.\n", totalOffs);
  } else {
    printf("LevelCompressor: Error opening 'compressedlevels.h' file!\n");
  }
  return 0;
}